# -*- coding: utf-8 -*-
# Insync Plasmoid 0.0.1
# Copyright Insync 2012
# License: GNU GPL

import os, sys
from PyQt4.QtCore import Qt, QObject, SIGNAL, QString, QUrl, QTimer
from PyQt4.QtGui import QGraphicsLinearLayout, QGraphicsGridLayout
from PyQt4.QtGui import QIcon, QMenu, QWidget, QPushButton, QGraphicsScene
from PyKDE4.plasma import Plasma
from PyKDE4 import plasmascript
from PyKDE4 import kdecore


#sys.path.append( '/usr/share/kde4/apps/plasma_scriptengine_python/' )
#import pyappletscript


import select, socket, json

DOMAIN = u'/tmp/insync-plasmoid%r.sock' % os.getuid()
if os.path.exists(DOMAIN):
  os.unlink(DOMAIN)

def ipc_plasmoid( **kw ):
  data = json.dumps( kw )
  socket_file = DOMAIN
  if not os.path.exists(socket_file): return None
  sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
  sock.connect(socket_file)
  sock.send(data)
  res = sock.recv(4096)
  sock.close()
  return res

import SocketServer
class Server(SocketServer.UnixStreamServer):
  def update(self): self.handle_request()

  def __init__(self, widget):
    SocketServer.UnixStreamServer.__init__(self, DOMAIN, self.myhandler, bind_and_activate=True)
    self.timeout = 0.1
    self.widget = widget

  def myhandler(self, request, client_address, _server):
    data = request.recv(4096).strip()
    print('raw data', data)
    msg = json.loads(data)
    assert isinstance(msg, dict)
    mode = msg['method']
    if mode == 'state':
      self.widget.title.setText('Insync (%s)'%msg['state'])
    elif mode == 'status':
      path = msg['path']
      status = msg['status']
      self.widget.update_status( path, status )

    else:
      print('unknown message type')

  def update(self):
    self.handle_request()

class InsyncWidget(plasmascript.Applet):
  def __init__(self,parent,args=None):
    plasmascript.Applet.__init__(self,parent)
    self.server = Server( self )
    self._labels = []
    self._updates = {}
    self._animations = {}

  def init(self, size=15):
    print('APPLET', self.applet)
    self.setHasConfigurationInterface(False)
    self.setAspectRatioMode(Plasma.IgnoreAspectRatio)

    self.theme = Plasma.Svg(self)
    self.theme.setImagePath("widgets/background")
    self.setBackgroundHints(Plasma.Applet.DefaultBackground)

    #self.layout = QGraphicsLinearLayout(Qt.Vertical, self.applet)
    self.layout = QGraphicsGridLayout(self.applet)
    self.layout.setColumnMinimumWidth(0, 16)
    self.layout.setColumnMaximumWidth(0, 16)

    self.title = label = Plasma.Label(self.applet)
    #label.setRotation( 45 )

    label.setText('Insync Plasmoid')
    #self.layout.addItem(label)
    self.layout.addItem(label, 0, 1)

    for i in range( size ):
      self.layout.setRowMinimumHeight( i+1, 16 )
      #icon = Plasma.Label(self.applet)
      icon = Plasma.IconWidget( self.applet )
      #point = icon.transformOriginPoint()
      #print( point.x(), point.y() )
      #point.setX( 8 )                  # TODO - this is not correct
      #point.setY( 8 )
      #icon.setTransformOriginPoint( point )
      self.layout.addItem(icon, i+1, 0)
      label = Plasma.Label(self.applet)
      self.layout.addItem(label, i+1, 1)
      self._labels.append( (icon,label) )

    self._labels.reverse()

    self.applet.setLayout(self.layout)
    self.resize(200,400)

    second = 1000

    self._timer = QTimer()
    self.connect(self._timer,SIGNAL("timeout()"), self.server.update)
    self._timer.start( second )

    self._animation_timer = QTimer()
    self.connect(self._timer,SIGNAL("timeout()"), self.update_animations)
    self._timer.start( second*0.1 )


  EMBLEM_KEYS = {
    'SYNCED': 'insync-synced',
    'SYNCING': 'insync-syncing',
    'ERROR': 'insync-error',
    'DES_ERROR': 'insync-des-error',
    'DES_SYNCING': 'insync-syncing',
  }

  def update_status(self, path, status):
    name = os.path.split(path)[-1]

    if path not in self._updates:
      a = self._labels.pop()
      self._updates[ path ] = a
      self._labels.insert(0, a)
      icon, label = a
      label.setText( name )

    icon, label = self._updates[ path ]
    assert status in self.EMBLEM_KEYS
    #icon.setImage( '/usr/share/icons/insync/emblem-%s.png'%self.EMBLEM_KEYS[status] )
    icon.setIcon( '/usr/share/icons/insync/emblem-%s.png'%self.EMBLEM_KEYS[status] )
    if status == 'SYNCING':
      self._animations[ path ] = icon
    elif status == 'SYNCED' and path in self._animations:
      self._animations.pop( path )
      icon.setRotation(0)


  def update_animations(self):
    if not self._animations: return
    for icon in self._animations.values():
      icon.setRotation( icon.rotation()+1 )



def CreateApplet(parent):
  return InsyncWidget(parent)



